﻿//
// PeerActivity.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Globalization;

namespace Ntp.Analyzer.Objects
{
    public sealed class PeerActivity : PersistentObject
    {
        public PeerActivity(int id, Peer peer, Host host, DateTime lastActive, bool hide)
            : base(id)
        {
            this.peer = peer;
            this.host = host;
            this.hide = hide;
            this.lastActive = lastActive;
        }

        public PeerActivity(Peer peer, Host host, DateTime lastActive)
        {
            this.peer = peer;
            this.host = host;
            this.lastActive = lastActive;
        }

        private readonly bool hide;

        private readonly Host host;
        private readonly Peer peer;
        private DateTime lastActive;

        public string Name
        {
            get { return Id.ToString(CultureInfo.InvariantCulture); }
        }

        public Peer Peer
        {
            get { return peer; }
        }

        public Host Host
        {
            get { return host; }
        }

        public DateTime LastActive
        {
            get { return lastActive; }
            set { lastActive = value; }
        }

        public bool Hide
        {
            get { return hide; }
        }

        public bool IsActive
        {
            get { return !hide; // && Math.Abs(DateTime.Now.Subtract (lastActive).TotalMinutes) <= 120.0;
            }
        }
    }
}