//
// DropdownItemConfiguration.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using Ntp.Analyzer.Localize;
using Ntp.Config;

namespace Ntp.Analyzer.Config.Navigation
{
    public sealed class DropdownItemConfiguration : MenuItemConfiguration
    {
        private DropdownItemConfiguration(
            string configName,
            int indentLevel,
            string itemName,
            string caption,
            MenuConfiguration menu)
            : base(configName, indentLevel, caption, null)
        {
            this.menu = menu;
        }

        private readonly String[] Params =
        {
            ConfigurationKeyword.MenuItem,
            ConfigurationKeyword.MenuItemType,
            ConfigurationKeyword.MenuItemCaption,
            ConfigurationKeyword.MenuItemLink,
            ConfigurationKeyword.MenuItemPage,
            ConfigurationKeyword.MenuItemDropdown
        };

        private readonly MenuConfiguration menu;

        public override string TypeText
        {
            get { return ConfigurationKeyword.MenuItemDropdown; }
        }

        public IEnumerable<MenuItemConfiguration> MenuItems
        {
            get { return menu.MenuItems; }
        }

        public override ILinkable Page
        {
            get { return null; }
        }

        protected override IEnumerable<Object> Items
        {
            get
            {
                return new object[]
                {
                    ConfigName,
                    TypeText,
                    Caption,
                    Link,
                    Page != null ? Page.Name : null,
                    MenuItems
                };
            }
        }

        public override string ToString()
        {
            return ConfigText(Params, Items);
        }

        public static MenuItemConfiguration Create(
            string itemName,
            string caption,
            ConfigurationBlock config,
            IEnumerable<ILinkable> linkables)
        {
            if (config == null || linkables == null)
                return null;

            MenuConfiguration menu = MenuConfiguration.Create(config, linkables);
            return new DropdownItemConfiguration(String.Empty, config.IndentLevel, itemName, caption, menu);
        }
    }
}