//
// DirectoryStreamDestination.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.IO;
using Ntp.Log;

namespace Ntp.Analyzer.Export
{
    public class DirectoryStreamDestination : FileSystemDestination
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Ntp.Analyzer.Export.DirectoryStreamDestination" /> class.
        /// </summary>
        /// <param name="basePath">Base path.</param>
        /// <param name="path">Path.</param>
        /// <param name="prefix">Prefix.</param>
        public DirectoryStreamDestination(string basePath, string path, string prefix)
        {
            path = (path != null ? path.Trim() : String.Empty);
            basePath = (basePath != null ? basePath.Trim() : String.Empty);

            if (basePath.Length > 1 && path[path.Length - 1] != '/')
                basePath = basePath + "/";

            if (path.Length > 1 && path[path.Length - 1] != '/')
                path = path + "/";

            this.path = basePath + path;
            this.prefix = prefix;
            value = path;
        }

        private readonly string path;
        private readonly string prefix;
        private readonly string value;

        public override string Link
        {
            get { return (value + "/" + prefix).Replace("//", "/"); }
        }

        public override string Value
        {
            get { return value; }
        }

        public override string Location
        {
            get { return path; }
        }

        public override string AbsoluteLocation
        {
            get { return path + prefix; }
        }

        public override void Test()
        {
            TestFile(
                AbsoluteLocation + "configtest",
                "DirectoryStreamDestinationTest");
        }

        public override void Write(Stream stream, LogBase log)
        {
            string file = path + prefix + Param;

            PrepareFile(file, log);
            WriteFile(stream, file, log);
            ApplyPermissions(file, log);
        }
    }
}