//
// DirectStatusBuilder.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System.Collections.Generic;
using System.Linq;
using Ntp.Analyzer.Import;
using Ntp.Analyzer.Objects;
using Ntp.Log;

namespace Ntp.Analyzer.Statistics
{
    public sealed class DirectStatusBuilder : StatusBuilder
    {
        public DirectStatusBuilder(Host host, ServerName server, ServerType type, LogBase log)
            : base(host)
        {
            this.server = server;
            this.type = type;
            this.log = log;
        }

        private readonly LogBase log;

        private readonly ServerName server;
        private readonly ServerType type;

        protected override IList<AssociationEntry> PeerAssociations
        {
            get
            {
                Importer<AssociationEntry> importer = ImportFactory.CreatePeerImporter(server.Name, type, Host, log);
                return importer.ToList();
            }
        }
    }
}